/*
================================================================================
  RPL/2 version 4.00
    Interprteur du langage de programmation du calculateur HP-28S,

    Date de cration    : 02 Avril 1.998

    Tous droits rservs  l'auteur, Jol BERTRAND
================================================================================
*/

/*
================================================================================
  Copyright (C) 2002, 2003, 2004 BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
 
  You should have received a copy of the GNU General Public License
  along with RPL/2; see the file COPYING.  If not, write to the Free
  Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
================================================================================
*/


#ifndef INCLUSION_RPL
#define INCLUSION_RPL

/*
================================================================================
  INCLUSIONS
================================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#ifndef RPLARGS
#	include <sys/resource.h>
#	include <sys/select.h>
#	include <sys/stat.h>
#	include <sys/time.h>
#	include <sys/timeb.h>
#	include <sys/types.h>
#	include <sys/wait.h>

#	include <dlfcn.h>
#	include <fcntl.h>
#	include <pthread.h>
#	include <pwd.h>
#	include <setjmp.h>
#	include <signal.h>
#	include <termios.h>
#	include <time.h>
#	include <unistd.h>

#	include "history.h"
#	include "readline.h"
#	include "termcap.h"

#	include "gsl/gsl_cdf.h"
#	include "gsl/gsl_errno.h"
#	include "gsl/gsl_fft_complex.h"
#	include "gsl/gsl_randist.h"
#	include "gsl/gsl_rng.h"
#	include "gsl/gsl_sf.h"
#endif


/*
================================================================================
  SIGNAUX
================================================================================
*/

#ifndef RPLARGS
#	define SIGSTART				SIGRTMIN+4
#endif

/*
================================================================================
  Granularit temporelle
================================================================================
*/

#ifndef RPLARGS
#	define GRANULARITE_us			10
#endif

/*
================================================================================
  Gnrateur alatoire
================================================================================
*/

#ifndef RPLARGS
#	define GENERATEUR_ALEATOIRE	gsl_rng_ranlux389
#endif

/*
================================================================================
  Affichage des points de retour des fonctions
================================================================================
*/

#ifdef DEBUG
#define return printf("Return from <%s()> at line #%d\n", __FUNCTION__, \
		__LINE__); return
#endif

/*
================================================================================
  TYPES
================================================================================
*/

typedef FILE					file;
typedef unsigned char			t_8_bits;

#ifndef RPLARGS
#	include "rpltypes.conv.h"
#else
#	include "rpltypes.h"
#endif

/*
================================================================================
  Macros
================================================================================
*/

/* Redfinition de abs pour un fonctionnement en entier de type long long int */
/* Attention aux effets de bord de abs(i++) !                                 */
#define abs(i) (((i) >= 0) ? (i) : (-i))

/*
================================================================================
  DEFINIONS
================================================================================
*/

/*
--------------------------------------------------------------------------------
  Valeurs
--------------------------------------------------------------------------------
*/

#define d_faux					((logical1) 0L)
#define d_vrai					((logical1) -1L)

#define d_absence_erreur		((logical1) d_faux)
#define d_erreur				((logical1) d_vrai)

#define d_code_fin_chaine		'\0'
#define d_code_espace			' '
#define d_code_retour_chariot	'\n'
#define d_code_tabulation		'\t'

#define d_INT					"integer"
#define	d_REL					"real"
#define d_CPL					"complex"
#define d_VIN					"[ integer ]"
#define d_VRL					"[ real ]"
#define d_VCX					"[ complex ]"
#define d_MIN					"[[ integer ]]"
#define d_MRL					"[[ real ]]"
#define d_MCX					"[[ complex ]]"
#define d_BIN					"# binary[bodh]"
#define d_NOM					"'name'"
#define d_CHN					"\"string\""
#define d_LST					"{ list }"
#define d_ALG					"'algebraic'"
#define d_RPN					"<< RPN >>"
#define d_FCH					"File $n { format }"
#define d_SLB					"Library $n"
#define d_PRC					"Process $n"
#define d_SCK					"Socket $n"

/*
--------------------------------------------------------------------------------
  Paramtres du systme
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
#	define ds_beep								""
#	define ds_fichier_historique				".RPL2_history"
#	define ds_imposition_separateur_decimal		d_vrai
#	define ds_longueur_historique				256
#	define ds_nombre_maximum_fichiers_ouverts	65536
#	define ds_preprocesseur						"rplpp " \
													"--nostdinc -C 2> /dev/null"
#	define ds_rplcore							"rpl-core"
#	define ds_sdat								"SDAT"
#endif

/*
--------------------------------------------------------------------------------
  Erreurs systeme
--------------------------------------------------------------------------------
*/

// -1 : erreur provoque par une bibliothque externe
#ifndef RPLARGS
#	define d_es									0
#	define d_es_allocation_memoire				1
#	define d_es_variable_introuvable			2
#	define d_es_pile_vide						3
#	define d_es_end_incoherent					4
#	define d_es_peripherique_stdin				5
#	define d_es_erreur_fichier					6
#	define d_es_execution_bibliotheque			7
#	define d_es_signal							8
#	define d_es_processus						9
#	define d_es_pile_operationnelle_vide		10
#	define d_es_nombre_max_descripteurs			11
#endif

/*
--------------------------------------------------------------------------------
  Exceptions
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
#	define d_ep									0
#	define d_ep_division_par_zero				1
#	define d_ep_matrice_non_inversible			2
#	define d_ep_resultat_indefini				3
#	define d_ep_underflow						4
#	define d_ep_overflow						5
#	define d_ep_domaine_definition				6
#	define d_ep_perte_precision					7
#endif

/*
--------------------------------------------------------------------------------
  Erreurs de compilation
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
#	define d_ec									0
#	define d_ec_niveau_definition_negatif		1
#	define d_ec_nom_definition_invalide			2
#	define d_ec_erreur_boucle_definie			3
#	define d_ec_erreur_instruction_end			4
#	define d_ec_erreur_instruction_until		5
#	define d_ec_source_incoherent				6
#	define d_ec_erreur_instruction_while		7
#	define d_ec_erreur_instruction_then			8
#endif

/*
--------------------------------------------------------------------------------
  Erreurs  l'excution
--------------------------------------------------------------------------------
*/

// -1 : erreur provoque par une bibliothque externe
#ifndef RPLARGS
#	define d_ex									0
#	define d_ex_pile_vide						1
#	define d_ex_manque_argument					2
#	define d_ex_argument_invalide				3
#	define d_ex_erreur_type_argument			4
#	define d_ex_nom_reserve						5
#	define d_ex_nombre_arguments				6
#	define d_ex_dimensions_invalides			7
#	define d_ex_expression_invalide				8
#	define d_ex_depassement_de_capacite			9
#	define d_ex_erreur_traitement_condition		10
#	define d_ex_erreur_traitement_boucle		11
#	define d_ex_variable_non_definie			12
#	define d_ex_drapeau_inexistant				13
#	define d_ex_nom_invalide					14
#	define d_ex_element_inexistant				15
#	define d_ex_absence_variable				16
#	define d_ex_erreur_evaluation				17
#	define d_ex_diagonalisation_impossible		18
#	define d_ex_longueur_fft					19
#	define d_ex_queue_impression				20
#	define d_ex_type_trace_invalide				21
#	define d_ex_erreur_fichier					22
#	define d_ex_absence_equation				23
#	define d_ex_erreur_bibliotheque				24
#	define d_ex_execution_bibliotheque			25
#	define d_ex_matrice_non_definie_positive	26
#	define d_ex_matrice_statistique_invalide	27
#	define d_ex_dimensions_matrice_statistique	28
#	define d_ex_absence_observations			29
#	define d_ex_statistiques_echantillon		30
#	define d_ex_observations_inexistantes		31
#	define d_ex_syntaxe							32
#	define d_ex_cycle_hors_boucle				33
#	define d_ex_conversion_unite				34
#	define d_ex_erreur_parametre_fichier		35
#	define d_ex_erreur_acces_fichier			36
#	define d_ex_erreur_requete_fichier			37
#	define d_ex_erreur_format_fichier			38
#	define d_ex_fichier_verrouille				39
#	define d_ex_verrou_indefini					40
#	define d_ex_fin_de_fichier_atteinte			41
#	define d_ex_debut_de_fichier_atteint		42
#	define d_ex_erreur_type_fichier				43
#	define d_ex_fichier_vide					44
#	define d_ex_dimensions_differentes			45
#	define d_ex_routines_mathematiques			46
#	define d_ex_exit_hors_boucle				47
#	define d_ex_longueur_dft					48
#	define d_ex_absence_processus_pere			49
#	define d_ex_processus						50
#	define d_ex_image_processus					51
#	define d_ex_erreur_sql						52
#	define d_ex_variable_verrouillee			53
#	define d_ex_variable_volatile				54
#	define d_ex_erreur_processus				55
#	define d_ex_erreur_impression				56
#	define d_ex_nombre_arguments_fonction		57
#	define d_ex_fonction_indisponible			58
#	define d_ex_bibliotheque_chargee			59
#	define d_ex_aucun_symbole					60
#	define d_ex_definition_ambigue				61
#endif

/*
--------------------------------------------------------------------------------
  Erreurs de la ligne de commande
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
#	define d_os									0
#	define d_os_ligne_de_commande				1
#	define d_os_informations					2
#	define d_os_fichier_introuvable				3
#	define d_os_erreur_fichier					4
#endif

/*
================================================================================
  STRUCTURES
================================================================================
*/


/*
--------------------------------------------------------------------------------
  Structure objet
--------------------------------------------------------------------------------
*/

typedef struct objet
{
	unsigned char		type[4];

	/*
	ADR : adresse sur 32 bits (au moins) non signs (unsigned long)
			Adresse d'une dfinition  interprter.

	ALG : expression algbrique (struct_liste_chainee *)
			La liste chane contient la dfinition convertie en notation
			polonaise inverse. Ce type diffre du type 'RPN' au niveau
			du format de sortie (notation algbrique).

	BIN : entier binaire sur 64 bits (unsigned integer8 *)
			Sans objet.
			Type C/Fortran : unsigned integer8

	CHN : chaine de caracteres (character*(*), unsigned char *)
			Sans objet.

	CPL : complexe sur 2*64 bits (complex*16, struct_complexe16 *)
			Sans objet.
			Type C/Fortran : complex16

	FCH : descripteur de fichier (struct_fichier *).

	FCT : dclaration d'une fonction et de son nombre d'arguments
			(struct_fonction *). Ce type n'est ncessaire que dans le
			traitement des types 'ALG' et 'RPN'.

	INT : entier sur 64 bits (integer*8, integer8 *)
			Sans objet.
			Type C/Fortran : integer8

	LST : liste (struct_liste_chainee *)
			Les objets sont enregistrs sous forme de liste chane rcursive.
			Cela permet de traiter des tableaux de donnes htrognes de
			grandes dimensions. Le nombre de dimensions maximal est fix par
			la mmoire de la station.

	MCX : matrice de complexes (struct_matrice *)
			Sans objet.

	MIN : matrice d'entiers (struct_matrice *)
			Sans objet.

	MRL : matrice de reels (struct_matrice *)
			Sans objet.

	NOM : nom de variable (struct_nom *)
			Sans objet.

	PRC : processus (struct_processus_fils *)
			Sans objet.

	REL : reel sur 64 bits (real*8, real8 *)
			Sans objet.

	RPN : definition (struct_liste_chainee *)
			Ce type diffre du type 'ALG' au niveau du format de sortie
			(notation polonaise inverse).

	SLB : bibliothque dynamique partage (struct_bibliotheque *) 
			Sans objet.

	VCX : vecteur de complexes (struct_vecteur *)
			Sans objet.

	VIN : vecteur d'entiers (struct_vecteur *)
			Sans objet.

	VRL : vecteur de reels (struct_vecteur *)
			Sans objet.
	*/

	void				*objet;
} struct_objet;


/*
--------------------------------------------------------------------------------
  Structure fichier
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct fichier
{
	unsigned long				descripteur;

	unsigned char				acces;			/* S/D/I */
	unsigned char				binaire;		/* Y/N   */
	unsigned char				ouverture;
	unsigned char				protection;
	unsigned char				*nom;
	
	struct_objet				*format;

	integer8					position_clef;
} struct_fichier;
#endif


/*
--------------------------------------------------------------------------------
  Structure bibliothque
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct bibliotheque
{
	void						*descripteur;

	unsigned char				*nom;
} struct_bibliotheque;
#endif


/*
--------------------------------------------------------------------------------
  Structure descripteur fichier
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct descripteur_fichier
{
	unsigned long				identifiant;
	unsigned char				*nom;

	file						*descripteur;
} struct_descripteur_fichier;
#endif


/*
--------------------------------------------------------------------------------
  Structure liste chanee
--------------------------------------------------------------------------------
*/

typedef struct liste_chainee
{
	struct liste_chainee		*suivant;

	struct_objet				*donnee;
} struct_liste_chainee; 


/*
--------------------------------------------------------------------------------
  Structure contenant les fichiers graphiques (liste chane)
--------------------------------------------------------------------------------
*/

#ifndef RPLARLS
typedef struct fichier_graphique
{
	struct fichier_graphique	*suivant;

	logical1					presence_axes;

	unsigned char				*nom;
	unsigned char				type[21];

	int							dimensions;

	integer8					systeme_axes;
} struct_fichier_graphique;
#endif


/*
--------------------------------------------------------------------------------
  Structure fonction
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct fonction
{
	unsigned char				*nom_fonction;
	unsigned long				nombre_arguments;
} struct_fonction;
#endif


/*
--------------------------------------------------------------------------------
  Structure nom
--------------------------------------------------------------------------------
*/

typedef struct nom
{
	unsigned char				*nom;
	logical1					symbole;	/* d_vrai/d_faux */
	// symbole == d_vrai signifie que le nom a t introduit dans un
	// calcul sans les apostrophes.
} struct_nom;


/*
--------------------------------------------------------------------------------
  Structure processus fils
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct processus_fils
{
	pid_t						pid;

	int							pipe_erreurs[2];
	int							pipe_objets[2];
	int							pipe_nombre_objets_attente[2];
	integer8					nombre_objets_dans_pipe;
} struct_processus_fils;
#endif


/*
--------------------------------------------------------------------------------
  Structure liste pile systeme
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct liste_pile_systeme
{
	struct liste_pile_systeme	*suivant;

	struct_liste_chainee		*pointeur_objet_retour;

	struct_objet				*indice_boucle;
	struct_objet				*limite_indice_boucle;
	struct_objet				*objet_de_test;

	logical1					arret_si_exception;
	logical1					creation_variables_statiques;

	unsigned char				clause;

	/*
	clause :
	I : IF
	R : IFERR
	X : IFERR indiquant la prsence d'une erreur.
	T : THEN
	E : ELSE
	Z : traitement en cours de ELSE
	D : DO
	U : UNTIL
	W : WHILE
	M : WHILE avec une clause fausse.
	S : SELECT
	K : aucun cas CASE n'a encore t trait.
	C : au moins un cas de SELECT...CASE a t trait.
	Q : traitement en cours d'un cas CASE.
	F : traitement du cas DEFAULT
	*/

	unsigned char				retour_definition;
	unsigned char				*nom_variable;
	unsigned char				origine_routine_evaluation;
	unsigned char				type_cloture[6];

	/*
	type clture :
	CASE  : END termine un test SELECT/CASE/THEN/END/DEFAULT/END.
	DO    : END termine une boucle DO/UNTIL/END.
	IF    : END termine un test IF/THEN (ELSE)/END.
	IFERR : END termine un test IFERR/THEN (ELSE)/END.
	WHILE : END termine une boucle WHILE/REPEAT/END.

	FOR   : NEXT ou STEP termine une boucle avec compteur.
	START : NEXT ou STEP termine une boucle sans compteur.
	LOOP  : boucle utilis dans le traitement de l'instruction RETURN.
	        Elle correspond  une boucle FOR ou START mais sans son
			initialisation.
	*/

	unsigned long				adresse_retour;
	unsigned long				niveau_courant;
} struct_liste_pile_systeme;
#endif


/*
--------------------------------------------------------------------------------
  Structure variable
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct variable
{
	unsigned char			*nom;
	unsigned char			origine;

	unsigned long			niveau;
	unsigned long			variable_statique;

	struct_objet			*objet;

	logical1				variable_verrouillee;
} struct_variable;
#endif


/*
--------------------------------------------------------------------------------
  Structure variable statique
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct variable_statique
{
	unsigned char			*nom;
	unsigned char			origine;

	// Position de cration de la variable statique dans le programme ou dans
	// l'expression. 'origine' vaut 'P' pour programme ou 'E' pour expression.

	unsigned long			niveau;
	unsigned long			variable_statique;

	struct_objet			*objet;
} struct_variable_statique;
#endif


/*
--------------------------------------------------------------------------------
  Structure vecteur
--------------------------------------------------------------------------------
*/

typedef struct vecteur
{
	unsigned long		taille;

	unsigned char		type; /* C (complex*16), R (real*8), I (integer*8) */

	void				*tableau;
} struct_vecteur;


/*
--------------------------------------------------------------------------------
  Structure matrice
--------------------------------------------------------------------------------
*/

typedef struct matrice
{
	unsigned long		nombre_lignes;
	unsigned long		nombre_colonnes;

	unsigned char		type; /* C (complex*16), R (real*8), I (integer*8) */

	void				**tableau;
} struct_matrice;


/*
--------------------------------------------------------------------------------
  Structure arbre
--------------------------------------------------------------------------------
*/

typedef struct arbre
{
	struct_objet		*objet;

	unsigned long		nombre_feuilles;
	struct arbre		**feuilles;
} struct_arbre;


/*
--------------------------------------------------------------------------------
  Structures de gestion des instructions externes
--------------------------------------------------------------------------------
*/

typedef struct instruction_externe
{
	unsigned char		*nom;
	unsigned char		*nom_bibliotheque;
	void				*descripteur_bibliotheque;
} struct_instruction_externe;


/*
--------------------------------------------------------------------------------
  Structure argument (pour les fonctions dynamiques)
--------------------------------------------------------------------------------
*/

typedef struct rpl_arguments
{
/* Piles operationnelles								*/

	struct_liste_chainee		*l_base_pile;
	struct_liste_chainee		*l_base_pile_last;

/* Etat du calculateur									*/

	t_8_bits					drapeaux_etat[8];

/* Traitement des erreurs								*/

	unsigned char				*message_erreur;
	unsigned char				type_erreur;			/* S = systme
														   E = excution */
	integer8					erreur;

/* Traitement des messages d'aide						*/

	logical1					aide;					/* Affichage d'un
														   message d'aide
														   par la routine */

/* Nombre d'arguments									*/

	unsigned char				affichage_arguments;
	unsigned char				constante_symbolique;
	unsigned char				instruction_valide;
	unsigned char				test_instruction;

	integer8					nombre_arguments;
	// Nombre d'arguments de la fonction, positif ou nul.
} struct_rpl_arguments;


/*
--------------------------------------------------------------------------------
  Structure processus
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
typedef struct processus
{
/* Pointeurs sur le programme							*/

	unsigned char				*definitions_chainees;
	struct_liste_chainee		*expression_courante;

	unsigned char				*instruction_courante;
	struct_objet				*objet_courant;

	unsigned char				*nom_fichier_source;

	unsigned char				autorisation_empilement_programme;	/*Y/N*/
	unsigned char				autorisation_evaluation_nom;		/*Y/N*/
	unsigned char				instruction_valide;					/*Y/N*/
	unsigned char				instruction_intrinseque;			/*Y/I/N*/
	unsigned char				test_instruction;					/*Y/N*/
	unsigned char				affichage_arguments;				/*Y/N*/
	unsigned char				constante_symbolique;				/*Y/N*/
	unsigned char				traitement_symbolique;				/*Y/N*/
	unsigned char				mode_execution_programme;			/*Y/I/N*/
	unsigned char				mode_interactif;					/*Y/N*/
	unsigned char				mode_evaluation_expression;			/*Y/N*/
	unsigned char				traitement_cycle_exit;				/*N/E/C*/
	unsigned char				recherche_types_speciaux;			/*Y/N*/

	unsigned long				position_courante;
	unsigned long				longueur_definitions_chainees;

/* Requetes												*/

	unsigned char				requete_arret;						/*Y/N*/

/* Piles operationnelles								*/

	struct_liste_chainee		*l_base_pile;
	struct_liste_chainee		*l_base_pile_last;

	unsigned long				hauteur_pile_operationnelle;

/* Piles systemes										*/

	struct_liste_pile_systeme	*l_base_pile_systeme;

/* Gestion des processus                                */

	struct_liste_chainee		*l_base_pile_processus;

	logical1					presence_pipe;

	int							pipe_donnees;
	int							pipe_nombre_objets_attente;
	
/* Variables											*/

	struct_variable				*s_liste_variables;
	struct_variable_statique	*s_liste_variables_statiques;

	unsigned long				nombre_variables;
	unsigned long				nombre_variables_statiques;
	unsigned long				niveau_courant;
	unsigned long				position_variable_courante;
	unsigned long				position_variable_statique_courante;

	logical1					creation_variables_statiques;

/* Fichiers												*/

	struct_liste_chainee		*s_fichiers;

/* Bibliothques dynamiques								*/

	struct_liste_chainee		*s_bibliotheques;
	struct_instruction_externe	*s_instructions_externes;
	unsigned long				nombre_instructions_externes;

/* Fichier d'impression									*/

	unsigned char				*nom_fichier_impression;
	unsigned char				format_papier[16];

/* Parametres graphiques								*/

	struct_fichier_graphique	*fichiers_graphiques;	 /* NULL */

	unsigned char				type_trace_eq[21];

								/*
								 * 2D :
								 * FONCTION
								 * PARAMETRIQUE
								 * POLAIRE
								 * COURBES DE NIVEAU
								 *
								 * 3D :
								 * GRILLE 3D
								 * COURBE PARAMETRIQUE // A FAIRE
								 * SURFACE PARAMETRIQUE // A FAIRE
								 * CYNLINDRIQUE // A FAIRE
								 * SPHERIQUE // A FAIRE
								 */

	unsigned char				type_trace_sigma[21];

								/*
								 * POINTS
								 * HISTOGRAMME
								 * TABLE TRACANTE
								 */

								/*
								 * ECHELLES
								 */

	integer8					systeme_axes;

	real8						x_min;
	real8						x_max;
	real8						y_min;
	real8						y_max;
	real8						z_min;
	real8						z_max;

	real8						x_tics;
	real8						y_tics;
	real8						z_tics;
	
	integer8					mx_tics;
	integer8					my_tics;
	integer8					mz_tics;

	logical1					x_lines;
	logical1					y_lines;
	logical1					z_lines;

	logical1					mx_lines;
	logical1					my_lines;
	logical1					mz_lines;

								/*
								 * -1 : absent du graphique
								 *  0 : graduations automatiques
								 * >0 : valeur manuelle
								 */

	real8						x2_min;
	real8						x2_max;
	real8						y2_min;
	real8						y2_max;
	real8						z2_min;
	real8						z2_max;

	real8						x2_tics;
	real8						y2_tics;
	real8						z2_tics;

	integer8					mx2_tics;
	integer8					my2_tics;
	integer8					mz2_tics;

	logical1					x2_lines;
	logical1					y2_lines;
	logical1					z2_lines;

	logical1					mx2_lines;
	logical1					my2_lines;
	logical1					mz2_lines;

	real8						resolution;

	logical1					echelle_automatique_x;
	logical1					echelle_automatique_y;
	logical1					echelle_automatique_z;

	logical1					echelle_automatique_x2;
	logical1					echelle_automatique_y2;
	logical1					echelle_automatique_z2;

	logical1					echelle_log_x;
	logical1					echelle_log_y;
	logical1					echelle_log_z;

	logical1					echelle_log_x2;
	logical1					echelle_log_y2;
	logical1					echelle_log_z2;

	real8						point_de_vue_theta;
	real8						point_de_vue_phi;
	real8						echelle_3D;

	unsigned char				*label_x;
	unsigned char				*label_y;
	unsigned char				*label_z;
	unsigned char				*titre;

	unsigned char				*nom_fichier_gnuplot;
	unsigned char				*type_fichier_gnuplot;

	struct_objet				*indep;
	struct_objet				*depend;
	struct_objet				*parametres_courbes_de_niveau;

	file						*entree_standard;

/* Parametres statistiques								*/

	integer8					colonne_statistique_1;
	integer8					colonne_statistique_2;

/* Etat du calculateur									*/

	t_8_bits					drapeaux_etat[8];

/* Traitement des exceptions							*/

	logical1					arret_si_exception;
	unsigned int				exception;
	unsigned int				derniere_exception;
	unsigned int				exception_processus_fils;

/* Traitement des erreurs								*/

	unsigned int				erreur_systeme;
	unsigned int				erreur_compilation;
	unsigned int				erreur_execution;

	logical1					erreur_scrutation;

	unsigned int				erreur_systeme_processus_fils;
	unsigned int				erreur_execution_processus_fils;

	logical1					erreur_processus_fils;

	integer8					derniere_erreur_fonction_externe;

	unsigned int				derniere_erreur_systeme;
	unsigned int				derniere_erreur_execution;
	unsigned int				derniere_erreur_evaluation;

	unsigned char				*instruction_derniere_erreur;

	logical1					core;
	logical1					invalidation_message_erreur;

/* Debug												*/

	logical1					debug;
	logical1					debug_programme;
	logical1					execution_pas_suivant;
	logical1					traitement_instruction_halt;

/* Autres												*/

	gsl_rng						*generateur_aleatoire;

	integer8					nombre_arguments;
	/* Il s'agit du nombre d'arguments utilis pour l'valuation
	 * des expressions algbriques.
	 * -1 : la fonction ne peut tre dans un objet de type ALG.
	 * -2 : nombre d'arguments inconnu
	 *  0 : la fonction doit apparatre en notation infixe
	 * >0 : la fonction apparat comme une fonction normale */

	logical1					lancement_interactif;

	unsigned char				*chemin_fichiers_temporaires;
	unsigned char				langue;
	unsigned char				retour_routine_evaluation;

	unsigned long				niveau_recursivite;
} struct_processus;
#endif


/*
================================================================================
  PROTOTYPES
================================================================================
*/

#ifdef LIBRPL
#	include "librplprototypes.h"
#endif

/*
--------------------------------------------------------------------------------
  Procdures d'excution des mots-clef du langage RPL/2
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS

// BEGIN C PROTOTYPES
void instruction_abort(struct_processus *s_etat_processus);
void instruction_abs(struct_processus *s_etat_processus);
void instruction_acos(struct_processus *s_etat_processus);
void instruction_acosh(struct_processus *s_etat_processus);
void instruction_alog(struct_processus *s_etat_processus);
void instruction_and(struct_processus *s_etat_processus);
void instruction_append(struct_processus *s_etat_processus);
void instruction_array_fleche(struct_processus *s_etat_processus);
void instruction_arg(struct_processus *s_etat_processus);
void instruction_asin(struct_processus *s_etat_processus);
void instruction_asinh(struct_processus *s_etat_processus);
void instruction_asl(struct_processus *s_etat_processus);
void instruction_asr(struct_processus *s_etat_processus);
void instruction_atan(struct_processus *s_etat_processus);
void instruction_atanh(struct_processus *s_etat_processus);
void instruction_autoscale(struct_processus *s_etat_processus);
void instruction_axes(struct_processus *s_etat_processus);

void instruction_b_vers_r(struct_processus *s_etat_processus);
void instruction_backspace(struct_processus *s_etat_processus);
void instruction_beep(struct_processus *s_etat_processus);
void instruction_bessel(struct_processus *s_etat_processus);
void instruction_bin(struct_processus *s_etat_processus);

void instruction_c_vers_r(struct_processus *s_etat_processus);
void instruction_case(struct_processus *s_etat_processus);
void instruction_ceil(struct_processus *s_etat_processus);
void instruction_centr(struct_processus *s_etat_processus);
void instruction_cf(struct_processus *s_etat_processus);
void instruction_chr(struct_processus *s_etat_processus);
void instruction_clear(struct_processus *s_etat_processus);
void instruction_cllcd(struct_processus *s_etat_processus);
void instruction_clmf(struct_processus *s_etat_processus);
void instruction_close(struct_processus *s_etat_processus);
void instruction_cls(struct_processus *s_etat_processus);
void instruction_clusr(struct_processus *s_etat_processus);
void instruction_cnrm(struct_processus *s_etat_processus);
void instruction_cols(struct_processus *s_etat_processus);
void instruction_col_fleche(struct_processus *s_etat_processus);
void instruction_col_moins(struct_processus *s_etat_processus);
void instruction_col_plus(struct_processus *s_etat_processus);
void instruction_comb(struct_processus *s_etat_processus);
void instruction_con(struct_processus *s_etat_processus);
void instruction_cont(struct_processus *s_etat_processus);
void instruction_conj(struct_processus *s_etat_processus);
void instruction_convert(struct_processus *s_etat_processus);
void instruction_copyright(struct_processus *s_etat_processus);
void instruction_corr(struct_processus *s_etat_processus);
void instruction_cos(struct_processus *s_etat_processus);
void instruction_cosh(struct_processus *s_etat_processus);
void instruction_cov(struct_processus *s_etat_processus);
void instruction_cr(struct_processus *s_etat_processus);
void instruction_create(struct_processus *s_etat_processus);
void instruction_cross(struct_processus *s_etat_processus);
void instruction_cswp(struct_processus *s_etat_processus);
void instruction_cycle(struct_processus *s_etat_processus);

void instruction_d_vers_r(struct_processus *s_etat_processus);
void instruction_date(struct_processus *s_etat_processus);
void instruction_dec(struct_processus *s_etat_processus);
void instruction_decr(struct_processus *s_etat_processus);
void instruction_default(struct_processus *s_etat_processus);
void instruction_deg(struct_processus *s_etat_processus);
void instruction_delete(struct_processus *s_etat_processus);
void instruction_depnd(struct_processus *s_etat_processus);
void instruction_depth(struct_processus *s_etat_processus);
void instruction_der(struct_processus *s_etat_processus);
void instruction_det(struct_processus *s_etat_processus);
void instruction_detach(struct_processus *s_etat_processus);
void instruction_dft(struct_processus *s_etat_processus);
void instruction_disp(struct_processus *s_etat_processus);
void instruction_division(struct_processus *s_etat_processus);
void instruction_do(struct_processus *s_etat_processus);
void instruction_dot(struct_processus *s_etat_processus);
void instruction_draw(struct_processus *s_etat_processus);
void instruction_drax(struct_processus *s_etat_processus);
void instruction_drop(struct_processus *s_etat_processus);
void instruction_drop2(struct_processus *s_etat_processus);
void instruction_dropn(struct_processus *s_etat_processus);
void instruction_drws(struct_processus *s_etat_processus);
void instruction_dup(struct_processus *s_etat_processus);
void instruction_dup2(struct_processus *s_etat_processus);
void instruction_dupn(struct_processus *s_etat_processus);

void instruction_e(struct_processus *s_etat_processus);
void instruction_edit(struct_processus *s_etat_processus);
void instruction_egalite(struct_processus *s_etat_processus);
void instruction_egv(struct_processus *s_etat_processus);
void instruction_egvl(struct_processus *s_etat_processus);
void instruction_else(struct_processus *s_etat_processus);
void instruction_elseif(struct_processus *s_etat_processus);
void instruction_end(struct_processus *s_etat_processus);
void instruction_eng(struct_processus *s_etat_processus);
void instruction_epsilon(struct_processus *s_etat_processus);
void instruction_erase(struct_processus *s_etat_processus);
void instruction_errm(struct_processus *s_etat_processus);
void instruction_errn(struct_processus *s_etat_processus);
void instruction_eval(struct_processus *s_etat_processus);
void instruction_exit(struct_processus *s_etat_processus);
void instruction_exp(struct_processus *s_etat_processus);
void instruction_expm(struct_processus *s_etat_processus);
void instruction_externals(struct_processus *s_etat_processus);
void instruction_eyept(struct_processus *s_etat_processus);

void instruction_fact(struct_processus *s_etat_processus);
void instruction_false(struct_processus *s_etat_processus);
void instruction_fc_test(struct_processus *s_etat_processus);
void instruction_fc_test_c(struct_processus *s_etat_processus);
void instruction_fc_test_s(struct_processus *s_etat_processus);
void instruction_fft(struct_processus *s_etat_processus);
void instruction_fix(struct_processus *s_etat_processus);
void instruction_fleche(struct_processus *s_etat_processus);
void instruction_fleche_array(struct_processus *s_etat_processus);
void instruction_fleche_col(struct_processus *s_etat_processus);
void instruction_fleche_hms(struct_processus *s_etat_processus);
void instruction_fleche_lcd(struct_processus *s_etat_processus);
void instruction_fleche_list(struct_processus *s_etat_processus);
void instruction_fleche_q(struct_processus *s_etat_processus);
void instruction_fleche_row(struct_processus *s_etat_processus);
void instruction_fleche_str(struct_processus *s_etat_processus);
void instruction_floor(struct_processus *s_etat_processus);
void instruction_for(struct_processus *s_etat_processus);
void instruction_format(struct_processus *s_etat_processus);
void instruction_fp(struct_processus *s_etat_processus);
void instruction_fs_test(struct_processus *s_etat_processus);
void instruction_fs_test_c(struct_processus *s_etat_processus);
void instruction_fs_test_s(struct_processus *s_etat_processus);
void instruction_function(struct_processus *s_etat_processus);

void instruction_gamma(struct_processus *s_etat_processus);
void instruction_ge(struct_processus *s_etat_processus);
void instruction_gegv(struct_processus *s_etat_processus);
void instruction_gegvl(struct_processus *s_etat_processus);
void instruction_get(struct_processus *s_etat_processus);
void instruction_getc(struct_processus *s_etat_processus);
void instruction_geti(struct_processus *s_etat_processus);
void instruction_getr(struct_processus *s_etat_processus);
void instruction_glegv(struct_processus *s_etat_processus);
void instruction_gregv(struct_processus *s_etat_processus);
void instruction_gt(struct_processus *s_etat_processus);

void instruction_halt(struct_processus *s_etat_processus);
void instruction_help(struct_processus *s_etat_processus);
void instruction_hex(struct_processus *s_etat_processus);
void instruction_histogram(struct_processus *s_etat_processus);
void instruction_hms_fleche(struct_processus *s_etat_processus);
void instruction_hms_moins(struct_processus *s_etat_processus);
void instruction_hms_plus(struct_processus *s_etat_processus);

void instruction_i(struct_processus *s_etat_processus);
void instruction_idft(struct_processus *s_etat_processus);
void instruction_idn(struct_processus *s_etat_processus);
void instruction_if(struct_processus *s_etat_processus);
void instruction_iferr(struct_processus *s_etat_processus);
void instruction_ifft(struct_processus *s_etat_processus);
void instruction_ift(struct_processus *s_etat_processus);
void instruction_ifte(struct_processus *s_etat_processus);
void instruction_im(struct_processus *s_etat_processus);
void instruction_incr(struct_processus *s_etat_processus);
void instruction_indep(struct_processus *s_etat_processus);
void instruction_input(struct_processus *s_etat_processus);
void instruction_inquire(struct_processus *s_etat_processus);
void instruction_int(struct_processus *s_etat_processus);
void instruction_inv(struct_processus *s_etat_processus);
void instruction_ip(struct_processus *s_etat_processus);

void instruction_key(struct_processus *s_etat_processus);
void instruction_kill(struct_processus *s_etat_processus);
void instruction_kind(struct_processus *s_etat_processus);

void instruction_label(struct_processus *s_etat_processus);
void instruction_last(struct_processus *s_etat_processus);
void instruction_lcd_fleche(struct_processus *s_etat_processus);
void instruction_lchol(struct_processus *s_etat_processus);
void instruction_le(struct_processus *s_etat_processus);
void instruction_legv(struct_processus *s_etat_processus);
void instruction_list_fleche(struct_processus *s_etat_processus);
void instruction_ln(struct_processus *s_etat_processus);
void instruction_lnp1(struct_processus *s_etat_processus);
void instruction_lock(struct_processus *s_etat_processus);
void instruction_log(struct_processus *s_etat_processus);
void instruction_logscale(struct_processus *s_etat_processus);
void instruction_lsq(struct_processus *s_etat_processus);
void instruction_lt(struct_processus *s_etat_processus);
void instruction_lu(struct_processus *s_etat_processus);

void instruction_mant(struct_processus *s_etat_processus);
void instruction_max(struct_processus *s_etat_processus);
void instruction_maxs(struct_processus *s_etat_processus);
void instruction_mclrin(struct_processus *s_etat_processus);
void instruction_mean(struct_processus *s_etat_processus);
void instruction_mem(struct_processus *s_etat_processus);
void instruction_min(struct_processus *s_etat_processus);
void instruction_mins(struct_processus *s_etat_processus);
void instruction_mod(struct_processus *s_etat_processus);
void instruction_moins(struct_processus *s_etat_processus);
void instruction_multiplication(struct_processus *s_etat_processus);

void instruction_ne(struct_processus *s_etat_processus);
void instruction_neg(struct_processus *s_etat_processus);
void instruction_next(struct_processus *s_etat_processus);
void instruction_not(struct_processus *s_etat_processus);
void instruction_nrand(struct_processus *s_etat_processus);
void instruction_ns(struct_processus *s_etat_processus);
void instruction_num(struct_processus *s_etat_processus);

void instruction_oct(struct_processus *s_etat_processus);
void instruction_open(struct_processus *s_etat_processus);
void instruction_or(struct_processus *s_etat_processus);
void instruction_over(struct_processus *s_etat_processus);

void instruction_p_vers_r(struct_processus *s_etat_processus);
void instruction_paper(struct_processus *s_etat_processus);
void instruction_parameter(struct_processus *s_etat_processus);
void instruction_parametric(struct_processus *s_etat_processus);
void instruction_pcov(struct_processus *s_etat_processus);
void instruction_perm(struct_processus *s_etat_processus);
void instruction_persist(struct_processus *s_etat_processus);
void instruction_pi(struct_processus *s_etat_processus);
void instruction_pick(struct_processus *s_etat_processus);
void instruction_plotter(struct_processus *s_etat_processus);
void instruction_plus(struct_processus *s_etat_processus);
void instruction_pmax(struct_processus *s_etat_processus);
void instruction_pmin(struct_processus *s_etat_processus);
void instruction_polar(struct_processus *s_etat_processus);
void instruction_pos(struct_processus *s_etat_processus);
void instruction_pourcent(struct_processus *s_etat_processus);
void instruction_pourcent_ch(struct_processus *s_etat_processus);
void instruction_pourcent_t(struct_processus *s_etat_processus);
void instruction_ppar(struct_processus *s_etat_processus);
void instruction_pr1(struct_processus *s_etat_processus);
void instruction_print(struct_processus *s_etat_processus);
void instruction_prlcd(struct_processus *s_etat_processus);
void instruction_prmd(struct_processus *s_etat_processus);
void instruction_prompt(struct_processus *s_etat_processus);
void instruction_protect(struct_processus *s_etat_processus);
void instruction_prst(struct_processus *s_etat_processus);
void instruction_prstc(struct_processus *s_etat_processus);
void instruction_prusr(struct_processus *s_etat_processus);
void instruction_prvar(struct_processus *s_etat_processus);
void instruction_psdev(struct_processus *s_etat_processus);
void instruction_puissance(struct_processus *s_etat_processus);
void instruction_purge(struct_processus *s_etat_processus);
void instruction_put(struct_processus *s_etat_processus);
void instruction_putc(struct_processus *s_etat_processus);
void instruction_puti(struct_processus *s_etat_processus);
void instruction_putr(struct_processus *s_etat_processus);
void instruction_pvar(struct_processus *s_etat_processus);

void instruction_r_vers_b(struct_processus *s_etat_processus);
void instruction_r_vers_c(struct_processus *s_etat_processus);
void instruction_r_vers_d(struct_processus *s_etat_processus);
void instruction_r_vers_p(struct_processus *s_etat_processus);
void instruction_rad(struct_processus *s_etat_processus);
void instruction_rand(struct_processus *s_etat_processus);
void instruction_rceq(struct_processus *s_etat_processus);
void instruction_rci(struct_processus *s_etat_processus);
void instruction_rcij(struct_processus *s_etat_processus);
void instruction_rcl(struct_processus *s_etat_processus);
void instruction_rclf(struct_processus *s_etat_processus);
void instruction_rcls(struct_processus *s_etat_processus);
void instruction_rcws(struct_processus *s_etat_processus);
void instruction_rdm(struct_processus *s_etat_processus);
void instruction_rdz(struct_processus *s_etat_processus);
void instruction_re(struct_processus *s_etat_processus);
void instruction_read(struct_processus *s_etat_processus);
void instruction_recv(struct_processus *s_etat_processus);
void instruction_redraw(struct_processus *s_etat_processus);
void instruction_relax(struct_processus *s_etat_processus);
void instruction_remove(struct_processus *s_etat_processus);
void instruction_res(struct_processus *s_etat_processus);
void instruction_recall(struct_processus *s_etat_processus);
void instruction_regv(struct_processus *s_etat_processus);
void instruction_repeat(struct_processus *s_etat_processus);
void instruction_return(struct_processus *s_etat_processus);
void instruction_rewind(struct_processus *s_etat_processus);
void instruction_rl(struct_processus *s_etat_processus);
void instruction_rlb(struct_processus *s_etat_processus);
void instruction_rnd(struct_processus *s_etat_processus);
void instruction_rnrm(struct_processus *s_etat_processus);
void instruction_roll(struct_processus *s_etat_processus);
void instruction_rolld(struct_processus *s_etat_processus);
void instruction_rot(struct_processus *s_etat_processus);
void instruction_row_fleche(struct_processus *s_etat_processus);
void instruction_row_moins(struct_processus *s_etat_processus);
void instruction_row_plus(struct_processus *s_etat_processus);
void instruction_rr(struct_processus *s_etat_processus);
void instruction_rrb(struct_processus *s_etat_processus);
void instruction_rsd(struct_processus *s_etat_processus);
void instruction_rswp(struct_processus *s_etat_processus);

void instruction_s_moins(struct_processus *s_etat_processus);
void instruction_s_plus(struct_processus *s_etat_processus);
void instruction_same(struct_processus *s_etat_processus);
void instruction_save(struct_processus *s_etat_processus);
void instruction_scatter(struct_processus *s_etat_processus);
void instruction_scale(struct_processus *s_etat_processus);
void instruction_schur(struct_processus *s_etat_processus);
void instruction_sci(struct_processus *s_etat_processus);
void instruction_scls(struct_processus *s_etat_processus);
void instruction_sconj(struct_processus *s_etat_processus);
void instruction_sdev(struct_processus *s_etat_processus);
void instruction_select(struct_processus *s_etat_processus);
void instruction_send(struct_processus *s_etat_processus);
void instruction_sf(struct_processus *s_etat_processus);
void instruction_sign(struct_processus *s_etat_processus);
void instruction_sin(struct_processus *s_etat_processus);
void instruction_sinh(struct_processus *s_etat_processus);
void instruction_sinv(struct_processus *s_etat_processus);
void instruction_size(struct_processus *s_etat_processus);
void instruction_sl(struct_processus *s_etat_processus);
void instruction_slb(struct_processus *s_etat_processus);
void instruction_slice(struct_processus *s_etat_processus);
void instruction_slicescale(struct_processus *s_etat_processus);
void instruction_sneg(struct_processus *s_etat_processus);
void instruction_sort(struct_processus *s_etat_processus);
void instruction_spar(struct_processus *s_etat_processus);
void instruction_splash(struct_processus *s_etat_processus);
void instruction_sq(struct_processus *s_etat_processus);
void instruction_sql(struct_processus *s_etat_processus);
void instruction_sqrt(struct_processus *s_etat_processus);
void instruction_sr(struct_processus *s_etat_processus);
void instruction_srb(struct_processus *s_etat_processus);
void instruction_sst(struct_processus *s_etat_processus);
void instruction_star_d(struct_processus *s_etat_processus);
void instruction_star_h(struct_processus *s_etat_processus);
void instruction_star_s(struct_processus *s_etat_processus);
void instruction_star_w(struct_processus *s_etat_processus);
void instruction_start(struct_processus *s_etat_processus);
void instruction_static(struct_processus *s_etat_processus);
void instruction_std(struct_processus *s_etat_processus);
void instruction_step(struct_processus *s_etat_processus);
void instruction_steq(struct_processus *s_etat_processus);
void instruction_sto(struct_processus *s_etat_processus);
void instruction_sto_division(struct_processus *s_etat_processus);
void instruction_sto_fois(struct_processus *s_etat_processus);
void instruction_sto_moins(struct_processus *s_etat_processus);
void instruction_sto_plus(struct_processus *s_etat_processus);
void instruction_stof(struct_processus *s_etat_processus);
void instruction_stop(struct_processus *s_etat_processus);
void instruction_store(struct_processus *s_etat_processus);
void instruction_stos(struct_processus *s_etat_processus);
void instruction_str_fleche(struct_processus *s_etat_processus);
void instruction_stws(struct_processus *s_etat_processus);
void instruction_sub(struct_processus *s_etat_processus);
void instruction_suspend(struct_processus *s_etat_processus);
void instruction_swap(struct_processus *s_etat_processus);
void instruction_sx(struct_processus *s_etat_processus);
void instruction_sx2(struct_processus *s_etat_processus);
void instruction_sxy(struct_processus *s_etat_processus);
void instruction_sy(struct_processus *s_etat_processus);
void instruction_sy2(struct_processus *s_etat_processus);
void instruction_sync(struct_processus *s_etat_processus);
void instruction_syseval(struct_processus *s_etat_processus);

void instruction_tan(struct_processus *s_etat_processus);
void instruction_tanh(struct_processus *s_etat_processus);
void instruction_taylr(struct_processus *s_etat_processus);
void instruction_then(struct_processus *s_etat_processus);
void instruction_time(struct_processus *s_etat_processus);
void instruction_title(struct_processus *s_etat_processus);
void instruction_tot(struct_processus *s_etat_processus);
void instruction_trn(struct_processus *s_etat_processus);
void instruction_true(struct_processus *s_etat_processus);
void instruction_type(struct_processus *s_etat_processus);

void instruction_uchol(struct_processus *s_etat_processus);
void instruction_undo(struct_processus *s_etat_processus);
void instruction_unlock(struct_processus *s_etat_processus);
void instruction_unprotect(struct_processus *s_etat_processus);
void instruction_until(struct_processus *s_etat_processus);
void instruction_use(struct_processus *s_etat_processus);
void instruction_utpc(struct_processus *s_etat_processus);
void instruction_utpf(struct_processus *s_etat_processus);
void instruction_utpn(struct_processus *s_etat_processus);
void instruction_utpt(struct_processus *s_etat_processus);

void instruction_var(struct_processus *s_etat_processus);
void instruction_variable(struct_processus *s_etat_processus);
void instruction_vars(struct_processus *s_etat_processus);
void instruction_vers_niveau_inferieur(struct_processus *s_etat_processus);
void instruction_vers_niveau_superieur(struct_processus *s_etat_processus);
void instruction_version(struct_processus *s_etat_processus);
void instruction_visit(struct_processus *s_etat_processus);
void instruction_volatile(struct_processus *s_etat_processus);

void instruction_wait(struct_processus *s_etat_processus);
void instruction_warranty(struct_processus *s_etat_processus);
void instruction_wflock(struct_processus *s_etat_processus);
void instruction_wfproc(struct_processus *s_etat_processus);
void instruction_while(struct_processus *s_etat_processus);
void instruction_wireframe(struct_processus *s_etat_processus);
void instruction_write(struct_processus *s_etat_processus);

void instruction_xcol(struct_processus *s_etat_processus);
void instruction_xor(struct_processus *s_etat_processus);
void instruction_xpon(struct_processus *s_etat_processus);
void instruction_xroot(struct_processus *s_etat_processus);

void instruction_ycol(struct_processus *s_etat_processus);
#endif

/*
--------------------------------------------------------------------------------
  Procedures
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
void affichage_pile(struct_processus *s_etat_processus,
		struct_liste_chainee *l_element_courant, unsigned long
		niveau_courant);
void analyse(struct_processus *s_etat_processus);
void appel_gnuplot(struct_processus *s_etat_processus,
		unsigned char persistance);
void cf(struct_processus *s_etat_processus, unsigned char indice_drapeau);
void conversion_decimal_vers_hms(real8 *angle);
void conversion_degres_vers_radians(real8 *angle);
void conversion_element_tex(unsigned char **element, unsigned char *fonction);
void conversion_format(struct_processus *s_etat_processus,
		unsigned char *chaine);
void conversion_hms_vers_decimal(real8 *angle);
void conversion_radians_vers_degres(real8 *angle);
void correction_formateur_tex(struct_processus *s_etat_processus,
		unsigned char **ligne);
void depilement_pile_systeme(struct_processus *s_etat_processus);
void derivation(struct_processus *s_etat_processus,
		struct_objet **s_expression);
void ecriture_pile(struct_processus *s_etat_processus, file *flux,
		struct_liste_chainee *l_element_courant, unsigned long
		niveau_courant);
void encart(struct_processus *s_etat_processus, unsigned long duree);
void evaluation_romberg(struct_processus *s_etat_processus,
		struct_objet *s_expression, unsigned char *variable, real8 *point,
		real8 *valeur, logical1 *validite);
void factorisation_cholesky(struct_matrice *s_matrice, unsigned char mode,
		long *ios);
void factorisation_lu(struct_matrice *s_matrice, struct_matrice **s_permutation,
		long *ios);
void factorisation_schur(struct_matrice *s_matrice, struct_matrice **s_schur,
		long *ios);
void formateur_tex(struct_processus *s_etat_processus, struct_objet *s_objet,
		unsigned char environnement);
void formateur_elementaire_tex(struct_processus *s_etat_processus,
		file *fichier, struct_objet *s_objet, unsigned char environnement);
void empilement_pile_systeme(struct_processus *s_etat_processus);
void impression_pile(struct_processus *s_etat_processus,
			struct_liste_chainee *l_element_courant, unsigned char methode,
			unsigned long niveau_courant);
void impression_tex(struct_processus *s_etat_processus);
void informations(unsigned char langue);
void initialisation_completion(void);
void initialisation_drapeaux(struct_processus *s_etat_processus);
void integrale_romberg(struct_processus *s_etat_processus,
		struct_objet *s_expression, unsigned char *variable,
		real8 a, real8 b, real8 precision);
void interruption(int signal);
void interruption2(int signal);
void interruption3(int signal);
void interruption4(int signal);
void interruption5(int signal);
void inversion_matrice(struct_matrice *s_matrice, long *ios);
#endif

void liberation(struct_objet *s_objet);

#ifndef RPLARGS
void moindres_carres(struct_matrice *s_matrice_a, struct_matrice *s_matrice_b,
		struct_matrice *s_matrice_x, long *ios);
void recherche_type(struct_processus *s_etat_processus);
void rplcore(struct_processus *s_etat_processus);
void sf(struct_processus *s_etat_processus, unsigned char indice_drapeau);
void *surveillance_processus(void *argument);
void swap(void *variable_1, void *variable_2, unsigned long taille_octets);
void traitement_asynchrone_exceptions_gsl(struct_processus *s_etat_processus);
void traitement_exceptions_gsl(const char *reason, const char *file,
		int line, int gsl_errno);
void tri_base_symboles_externes(struct_processus *s_etat_processus);
void tri_base_variables(struct_processus *s_etat_processus);
void tri_base_variables_statiques(struct_processus *s_etat_processus);
void tri_niveaux_variables(struct_variable *s_liste_variables,
		unsigned long borne_inferieure, unsigned long borne_superieure);
void tri_positions_variables_statiques(struct_variable_statique
		*s_liste_variables,
		unsigned long borne_inferieure, unsigned long borne_superieure);
void tri_vecteur(real8 *vecteur, unsigned long taille);
void valeurs_propres(struct_matrice *s_matrice,
		struct_vecteur *s_valeurs_propres,
		struct_matrice *s_vecteurs_propres_gauches,
		struct_matrice *s_vecteur_propres_droits,
		long *ios);
void valeurs_propres_generalisees(struct_matrice *s_matrice,
		struct_matrice *s_metrique,
		struct_vecteur *s_valeurs_propres,
		struct_matrice *s_vecteurs_propres_gauches,
		struct_matrice *s_vecteur_propres_droits,
		long *ios, long *erreur_division);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions de sommation de vecteur dans perte de prcision
--------------------------------------------------------------------------------
*/

real8 sommation_vecteur_reel(real8 *vecteur, unsigned long *taille,
		logical1 *erreur_memoire);
complex16 sommation_vecteur_complexe(complex16 *vecteur, unsigned long *taille,
		logical1 *erreur_memoire);

/*
--------------------------------------------------------------------------------
  Routine principale
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
int rplinit(int argc, char *argv[], unsigned char ***resultats);
#endif

/*
--------------------------------------------------------------------------------
  Routines de statistiques
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
real8 correlation_statistique(struct_matrice *s_matrice,
		unsigned long colonne_1, unsigned long colonne_2, logical1 *erreur);
real8 covariance_statistique(struct_matrice *s_matrice,
		unsigned long colonne_1, unsigned long colonne_2,
		unsigned char type, logical1 *erreur);
struct_vecteur *ecart_type_statistique(struct_matrice *s_matrice,
		unsigned char type);
struct_vecteur *moyenne_statistique(struct_matrice *s_matrice);
struct_vecteur *sommation_colonne_statistique(struct_matrice *s_matrice,
		unsigned long colonne);
struct_vecteur *sommation_produits_colonnes_statistiques(struct_matrice
		*s_matrice, unsigned long colonne_1, unsigned long colonne_2);
struct_vecteur *sommation_statistique(struct_matrice *s_matrice);
struct_vecteur *variance_statistique(struct_matrice *s_matrice,
		unsigned char type);
#endif

/*
--------------------------------------------------------------------------------
  Gestion des bibliothques
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
void *chargement_bibliotheque(struct_processus *s_etat_processus,
		unsigned char *bibliotheque);
logical1 execution_fonction_de_bibliotheque(struct_processus *s_etat_processus,
		unsigned char *nom_fonction, unsigned char *bibliothque);
logical1 retrait_bibliotheque(struct_processus *s_etat_processus,
		struct_bibliotheque *descripteur);
#endif

struct_liste_chainee *depilement_pile_operationnelle(struct_rpl_arguments
		*s_rpl_arguments, struct_objet **s_objet);
struct_liste_chainee *empilement_pile_operationnelle(struct_rpl_arguments
		*s_rpl_arguments, struct_objet *s_objet);
struct_liste_chainee *sauvegarde_arguments(struct_rpl_arguments
		*s_rpl_arguments, unsigned long nombre_arguments);

/*
--------------------------------------------------------------------------------
  Fonctions retournant un CHARACTER*(*)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
unsigned char *analyse_algebrique(struct_processus *s_etat_processus,
		unsigned char *chaine_algebrique, struct_liste_chainee **l_base_liste);
unsigned char *recherche_chemin_fichiers_temporaires();
unsigned char *compactage(unsigned char *chaine);
unsigned char *conversion_majuscule(unsigned char *chaine);
unsigned char *creation_nom_fichier(unsigned char *chemin);
unsigned char *extraction_chaine(unsigned char *chaine, unsigned long i,
		unsigned long j);
unsigned char *formateur(struct_processus *s_etat_processus, long offset,
		struct_objet *s_objet);
unsigned char *formateur_fichier(struct_processus *s_etat_processus,
		struct_objet *s_objet, struct_objet *s_format,
		long longueur, long longueur_champ, unsigned char format,
		unsigned char type, long *longueur_effective);
unsigned char *formateur_fichier_nombre(struct_processus *s_etat_processus,
		void *valeur_numerique, unsigned char type,
		long longueur, long longueur_champ, unsigned char format);
unsigned char *formateur_fichier_reel(struct_processus *s_etat_processus,
		void *valeur_numerique, unsigned char type,
		long longueur, long longueur_champ, unsigned char format);
unsigned char *formateur_nombre(struct_processus *s_etat_processus,
		void *valeur_numerique, unsigned char type);
unsigned char *formateur_reel(struct_processus *s_etat_processus,
		void *valeur_numerique, unsigned char type);
unsigned char *messages(struct_processus *s_etat_processus);
unsigned char *purification_chaine(unsigned char *chaine);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions retournant un INTEGER*1 (drapeau d'erreur)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
logical1 analyse_syntaxique(struct_processus *s_etat_processus);
logical1 creation_variable(struct_processus *s_etat_processus,
		struct_variable *s_variable,
		unsigned char autorisation_creation_variable_statique);
logical1 creation_variable_statique(struct_processus *s_etat_processus,
		struct_variable_statique *s_variable);
logical1 chainage(struct_processus *s_etat_processus);
logical1 compilation(struct_processus *s_etat_processus);
logical1 depilement(struct_processus *s_etat_processus,
		struct_liste_chainee **l_base_liste, struct_objet **s_objet);
logical1 destruction_fichier(unsigned char *nom_fichier);
logical1 empilement(struct_processus *s_etat_processus,
		struct_liste_chainee **l_base_liste, struct_objet *s_objet);
logical1 empilement_pile_last(struct_processus *s_etat_processus,
		unsigned long nombre_arguments);
logical1 evaluation(struct_processus *s_etat_processus, struct_objet *s_objet,
		unsigned char type_evaluation);
logical1 recherche_instruction_suivante(struct_processus *s_etat_processus);
logical1 retrait_variable(struct_processus *s_etat_processus,
		unsigned char *nom_variable, unsigned char type);
logical1 retrait_variable_statique(struct_processus *s_etat_processus,
		unsigned char *nom_variable, unsigned long position);
logical1 sequenceur(struct_processus *s_etat_processus);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions retournant un INTEGER*1 (drapeau)
--------------------------------------------------------------------------------
*/

logical1 depassement_addition(integer8 *a, integer8 *b, integer8 *resultat);
logical1 depassement_multiplication(integer8 *a, integer8 *b,
		integer8 *resultat);
logical1 depassement_puissance(integer8 *a, integer8 *b, integer8 *resultat);

#ifndef RPLARGS
logical1 ecriture_pipe(struct_processus *s_etat_processus, int pipe,
		struct_objet *s_objet);
logical1 recherche_variable(struct_processus *s_etat_processus,
		unsigned char *nom_variable);
logical1 recherche_variable_statique(struct_processus *s_etat_processus,
		unsigned char *nom_variable, unsigned long position,
		unsigned char origine);
logical1 test_cfsf(struct_processus *s_etat_processus,
		unsigned char indice_drapeau);
logical1 test_expression_rpn(unsigned char *chaine);
logical1 test_fonction(unsigned char *chaine);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions renvoyant un INTEGER*1 (longueur d'un entier binaire)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
unsigned char longueur_entiers_binaires(struct_processus *s_etat_processus);
unsigned char recherche_longueur_optimale(integer8 parametre);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions retournant un INTEGER*8 (valeur binaire non signe)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
logical8 masque_entiers_binaires(struct_processus *s_etat_processus);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions retournant un INTEGER*8 (entier standard RPL/2)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
integer8 occupation_memoire(struct_objet *s_objet);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions renvoyant une liste chane (pointeur sur *struct_liste_chainee)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
struct_liste_chainee *analyse_rpn(struct_processus *s_etat_processus,
		unsigned char *chaine_rpn);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions renvoyant un pointeur sur un fichier (file *)
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
file *creation_fichier_tex(struct_processus *s_etat_processus);
file *descripteur_fichier(struct_processus *s_etat_processus,
		struct_fichier *s_fichier);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions renvoyant un pointeur sur une *struct_objet
--------------------------------------------------------------------------------
*/

struct_objet *copie_objet(struct_objet *s_objet);

#ifndef RPLARGS
struct_objet *lecture_pipe(struct_processus *s_etat_processus, int pipe);
struct_objet *simplification(struct_processus *s_etat_processus,
		struct_objet *s_objet);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions grant les arbres de simplification
--------------------------------------------------------------------------------
*/

#ifndef RPLARGS
struct_arbre *creation_arbre(struct_processus *s_etat_processus,
		struct_objet **t_objets, unsigned long indice,
		unsigned long indice_maximal);
void liberation_arbre(struct_processus *s_etat_processus,
		struct_arbre *s_noeud);
void parcours_arbre(struct_processus *s_etat_processus, struct_arbre *s_noeud);
void simplification_arbre(struct_processus *s_etat_processus,
		struct_arbre *s_noeud);
#endif

/*
--------------------------------------------------------------------------------
  Fonctions grant la compltion automatique
--------------------------------------------------------------------------------
*/

char **completion_rpl();
char *generateur_commandes();
char **completion_matches();

// END C PROTOTYPES

/*
--------------------------------------------------------------------------------
  Arithmtique
--------------------------------------------------------------------------------
*/

void f77absc_(complex16 *c, real8 *resultat);

void f77additioncc_(complex16 *ca, complex16 *cb, complex16 *resultat);
void f77additionci_(complex16 *ca, integer8 *ib, complex16 *resultat);
void f77additioncr_(complex16 *ca, real8 *rb, complex16 *resultat);

void f77divisioncc_(complex16 *ca, complex16 *rb, complex16 *resultat);
void f77divisionci_(complex16 *ca, integer8 *ib, complex16 *resultat);
void f77divisioncr_(complex16 *ca, real8 *rb, complex16 *resultat);
void f77divisionic_(integer8 *ib, complex16 *cb, complex16 *resultat);
void f77divisionrc_(real8 *ra, complex16 *cb, complex16 *resultat);

void f77multiplicationcc_(complex16 *ca, complex16 *cb, complex16 *resultat);
void f77multiplicationci_(complex16 *ca, integer8 *ib, complex16 *resultat);
void f77multiplicationcr_(complex16 *ca, real8 *rb, complex16 *resultat);

void f77puissancecc_(complex16 *ca, complex16 *cb, complex16 *resultat);
void f77puissanceci_(complex16 *ca, integer8 *ib, complex16 *resultat,
		integer4 *troncature);
void f77puissancecr_(complex16 *ca, real8 *rb, complex16 *resultat);
void f77puissanceic_(integer8 *ia, complex16 *cb, complex16 *resultat);
void f77puissanceii_(integer8 *ia, integer8 *ib, integer8 *resultat);
void f77puissanceir_(integer8 *ia, real8 *rb, real8 *resultat);
void f77puissancerc_(real8 *ra, complex16 *cb, complex16 *resultat);
void f77puissanceri_(real8 *ra, integer8 *ib, real8 *resultat,
		integer4 *troncature);
void f77puissancerr_(real8 *ra, real8 *rb, real8 *resultat);

void f77racinecarreec_(complex16 *ca, complex16 *resultat);
void f77racinecarreein_(integer8 *ia, complex16 *resultat);
void f77racinecarreeip_(integer8 *ia, real8 *resultat);
void f77racinecarreern_(real8 *ia, complex16 *resultat);
void f77racinecarreerp_(real8 *ia, real8 *resultat);

void f77soustractioncc_(complex16 *ca, complex16 *cb, complex16 *resultat);
void f77soustractionci_(complex16 *ca, integer8 *ib, complex16 *resultat);
void f77soustractioncr_(complex16 *ca, real8 *rb, complex16 *resultat);
void f77soustractionic_(integer8 *ib, complex16 *cb, complex16 *resultat);
void f77soustractionrc_(real8 *ra, complex16 *cb, complex16 *resultat);

/*
--------------------------------------------------------------------------------
  Trigonomtrie complexe
--------------------------------------------------------------------------------
*/

void f77acos_(complex16 *argument, complex16 *resultat);
void f77asin_(complex16 *argument, complex16 *resultat);
void f77atan_(complex16 *argument, complex16 *resultat, integer4 *erreur);
void f77cos_(complex16 *argument, complex16 *resultat);
void f77sin_(complex16 *argument, complex16 *resultat);
void f77tan_(complex16 *argument, complex16 *resultat, integer4 *erreur);

/*
--------------------------------------------------------------------------------
  Fonctions hyperboliques
--------------------------------------------------------------------------------
*/

void f77acoshc_(complex16 *argument, complex16 *resultat);
void f77alogc_(complex16 *argument, complex16 *resultat);
void f77asinhc_(complex16 *argument, complex16 *resultat);
void f77atanhc_(complex16 *argument, complex16 *resultat);
void f77cosh_(complex16 *argument, complex16 *resultat);
void f77expc_(complex16 *argument, complex16 *resultat);
void f77lnc_(complex16 *argument, complex16 *resultat, integer4 *erreur);
void f77lnin_(integer8 *argument, complex16 *resultat, integer4 *erreur);
void f77lnrn_(real8 *argument, complex16 *resultat, integer4 *erreur);
void f77logc_(complex16 *argument, complex16 *resultat, integer4 *erreur);
void f77login_(integer8 *argument, complex16 *resultat, integer4 *erreur);
void f77logrn_(real8 *argument, complex16 *resultat, integer4 *erreur);
void f77sinh_(complex16 *argument, complex16 *resultat);
void f77tanh_(complex16 *argument, complex16 *resultat, integer4 *erreur);
void f77acoshi_(integer8 *argument, real8 *resultat);
void f77acoshr_(real8 *argument, real8 *resultat);
void f77asinhi_(integer8 *argument, real8 *resultat);
void f77asinhr_(real8 *argument, real8 *resultat);
void f77atanhi_(integer8 *argument, real8 *resultat);
void f77atanhr_(real8 *argument, real8 *resultat);
void f77lnip_(integer8 *argument, real8 *resultat, integer4 *erreur);
void f77lnrp_(real8 *argument, real8 *resultat, integer4 *erreur);
void f77logip_(integer8 *argument, real8 *resultat, integer4 *erreur);
void f77logrp_(real8 *argument, real8 *resultat, integer4 *erreur);

/*
--------------------------------------------------------------------------------
  Calculs divers
--------------------------------------------------------------------------------
*/

void f90arrangement_(integer8 *n, integer8 *k, real8 *resultat);
void f90combinaison_(integer8 *n, integer8 *k, real8 *resultat);
void f90fcd_(real8 *x, integer8 *n1, integer8 *n2, real8 *resultat);
void f90gausscd_(real8 *x, real8 *moyenne, real8 *variance, real8 *resultat);
void f90tcd_(real8 *x, integer8 *n, real8 *resultat);
void f90x2cd_(real8 *x, integer8 *n, real8 *resultat);

// BEGIN C PROTOTYPES
void dft(complex16 *dft, integer4 *nombre_lignes, integer4 *nombre_colonnes,
		integer4 *inverse, integer4 *erreur);
// END C PROTOTYPES

/*
--------------------------------------------------------------------------------
  Fonctions Lapack
--------------------------------------------------------------------------------
*/

	/*
	 * Calcul de la longueur d'un espace de travail
	 */

#ifndef RPLARGS
integer4 ilaenv_(integer4 *optimal, char *nom_fonction, char *option,
		integer4 *n1, integer4 *n2, integer4 *n3, integer4 *n4,
		integer4 longueur_nom_fonction, integer4 longueur_option);

	/*
	 * Inversion
	 */

void dgetri_(integer4 *ordre_a, real8 *a, integer4 *lda, integer4 *pivot,
		real8 *work, integer4 *dim_work, integer4 *ios);
void zgetri_(integer4 *ordre_a, complex16 *a, integer4 *lda, integer4 *pivot,
		complex16 *work, integer4 *dim_work, integer4 *ios);

	/*
	 * Valeurs propres
	 */

void zgeev_(unsigned char *calcul_vp_gauches, unsigned char *calcul_vp_droits,
		integer4 *dim_matrice1, complex16 *matrice_f77,
		integer4 *dim_matrice2, complex16 *valeurs_propres,
		complex16 *vpg_f77, integer4 *dim_matrice3,
		complex16 *vpd_f77, integer4 *dim_matrice4,
		complex16 *work, integer4 *lwork, real8 *rwork,
		integer4 *erreur, integer4 longueur_1, integer4 longueur_2);

	/*
	 * Valeurs propres gnralises
	 */

void zggev_(unsigned char *calcul_vp_gauches, unsigned char *calcul_vp_droits,
		integer4 *dim_matrice1, complex16 *matrice_f77,
		integer4 *dim_matrice2, complex16 *metrique_f77,
		integer4 *dim_matrice3, complex16 *alpha, complex16 *beta,
		complex16 *vpg_f77, integer4 *dim_matrice4,
		complex16 *vpd_f77, integer4 *dim_matrice5,
		complex16 *work, integer4 *lwork, real8 *rwork,
		integer4 *erreur, integer4 longueur_1, integer4 longueur_2);

	/*
	 * Moindres carrs
	 */

void dgelsd_(integer4 *nombre_lignes_a, integer4 *nombre_colonnes_a,
		integer4 *nombre_colonnes_b, real8 *matrice_a, integer4 *lda,
		real8 *matrice_b, integer4 *ldb, real8 *valeurs_singulieres,
		real8 *rcond, integer4 *rank, real8 *work, integer4 *lwork,
		integer4 *iwork, integer4 *info);
void zgelsd_(integer4 *nombre_lignes_a, integer4 *nombre_colonnes_a,
		integer4 *nombre_colonnes_b, complex16 *matrice_a, integer4 *lda,
		complex16 *matrice_b, integer4 *ldb, real8 *valeurs_singulieres,
		real8 *rcond, integer4 *rank, complex16 *work, integer4 *lwork,
		real8 *rwork, integer4 *iwork, integer4 *info);

	/*
	 * Moindres carrs gnralis
	 */


	/*
	 * Factorisations diverses
	 */

		/* LU de Crout */
void dgetrf_(integer4 *nb_lignes_a, integer4 *nb_colonnes_a, real8 *a,
		integer4 *lda, integer4 *pivot, integer4 *ios);
void zgetrf_(integer4 *nb_lignes_a, integer4 *nb_colonnes_a, complex16 *a,
		integer4 *lda, integer4 *pivot, integer4 *ios);

		/* LQ */
void dgelqf_();
void zgelqf_();

		/* QR */
void dgeqp3_();
void zgeqp3_();

		/* Cholesky */
void dpptrf_(unsigned char *mode, integer4 *n, complex16 *triangle,
		integer4 *ios, integer4 longueur);
void zpptrf_(unsigned char *mode, integer4 *n, complex16 *triangle,
		integer4 *ios, integer4 longueur);

		/* Schur */
void dgees_(unsigned char *calcul_vecteur, unsigned char *valeurs_triees,
		logical4 (*fonction_tri)(), integer4 *n, real8 *a,
		integer4 *lda, integer4 *sdim, real8 *wr, real8 *wi,
		real8 *vs, integer4 *ldvs, real8 *work, integer4 *lwork,
		logical4 *bwork, integer4 *info, integer4 l1, integer4 l2);
void zgees_(unsigned char *calcul_vecteur, unsigned char *valeurs_triees,
		logical4 (*fonction_tri)(), integer4 *n, complex16 *a,
		integer4 *lda, integer4 *sdim, complex16 *w,
		complex16 *vs, integer4 *ldvs, complex16 *work, integer4 *lwork,
		real8 *rwork, logical4 *bwork, integer4 *info,
		integer4 l1, integer4 l2);

/*
--------------------------------------------------------------------------------
  Fonctions Fortran-77 ne renvoyant rien
--------------------------------------------------------------------------------
*/

void f77caracteristiquesfichier_(integer4 *unite, integer4 *erreur,
		char *nom, integer4 longueur_nom);
void f77creationfichier_(integer4 *unite, integer4 *erreur,
		char *nom, integer4 longueur_nom);
void f77destructionfichier_(integer4 *unite, integer4 *erreur,
		char *nom, integer4 longueur_nom);
void f77ecriturefichier_(integer4 *unite, integer4 *erreur,
		char *nom, integer4 longueur_nom);
void f77fermeturefichier_(integer4 *unite, integer4 *erreur);
void f77lecturefichier_(integer4 *unite, integer4 *dernier_enregistrement,
		integer4 *erreur, char *nom, integer4 longueur_nom);
void f77nomfichier_(integer4 *unite, integer4 *erreur, char *nom,
		integer4 longueur_nom);
void f77ouverturefichier_(integer4 *unite, integer4 *erreur,
		char *nom, integer4 longueur_nom);
void f77rembobinagefichier_(integer4 *unite, integer4 *erreur);
void f77retourenregistrementprecedentfichier_(integer4 *unite,
		integer4 *erreur);
#endif


/*
================================================================================
  Variables globales
================================================================================
*/

#ifndef RPLARGS
#	ifndef MAIN_RPL
	extern sig_atomic_t				var_globale_requete_arret;
	extern sig_atomic_t				var_globale_requete_arret2;

	extern int						exception_gsl;
	extern int						var_globale_processus_pere;
	extern int						var_globale_recursivite;

	extern pthread_mutex_t			mutex;

	extern jmp_buf					contexte;
	extern jmp_buf					contexte_initial;
#	else
	sig_atomic_t					var_globale_requete_arret = 0;
	sig_atomic_t					var_globale_requete_arret2 = 0;

	int								exception_gsl = 0;
	int								var_globale_processus_pere = -1;
	int								var_globale_recursivite = 0;

	pthread_mutex_t					mutex = PTHREAD_MUTEX_INITIALIZER;

	jmp_buf							contexte;
	jmp_buf							contexte_initial;
#	endif
#endif

#endif

// vim: ts=4
