/*
================================================================================
  RPL/2 version 4.00
    Interprteur du langage de programmation du calculateur HP-28S,

    Date de cration    : 02 Avril 1.998

    Tous droits rservs  l'auteur, Jol BERTRAND
================================================================================
*/

/*
================================================================================
  Copyright (C) 2001 BERTRAND Jol

  This file is part of RPL/2.

  RPL/2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.
 
  RPL/2 is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
 
  You should have received a copy of the GNU General Public License
  along with RPL/2; see the file COPYING.  If not, write to the Free
  Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
================================================================================
*/

#ifndef INCLUSION_RPLARGS
#	define INCLUSION_RPLARGS

/*
================================================================================
  INCLUSIONS
================================================================================
*/

#	include <stdio.h>
#	include <stdlib.h>
#	include <math.h>
#	include <string.h>


#	define RPLARGS
#	include "rpl.h"

/*
================================================================================
  MACROS SPECIFIQUES
================================================================================
*/

/*
--------------------------------------------------------------------------------
  Types
--------------------------------------------------------------------------------
*/

#define declareObject(object) struct_objet *object=NULL
#define declareInteger(integer) integer8 integer
#define declareReal(real) real8 real
#define declareComplex(complex) complex16

/*
--------------------------------------------------------------------------------
  Constructeurs
--------------------------------------------------------------------------------
*/

#define HEADER \
	int				__constante; \
	logical1		__evaluation; \
	logical1		__validation_instruction = d_faux; \
	logical1		__presence_aide = d_faux; \
	logical1		__presence_validation = d_faux; \
	unsigned char	__indice_bit; \
	unsigned char	__indice_bloc; \
	unsigned char	__taille_bloc; \
	unsigned char	__type; \
	t_8_bits		__masque; \
	{ \
		(*rpl_arguments).instruction_valide = 'Y'; \
		(*rpl_arguments).erreur = 0; \
		__constante = 0; \
		__evaluation = d_faux;

#define FUNCTION \
	if (__validation_instruction == d_vrai) return; \
	if (__presence_aide == d_faux) \
	{ \
		systemError(__LINE__, "Help string not defined"); \
	}\
	else if (__presence_validation == d_faux) \
	{ \
		systemError(__LINE__, "Number of arguments not defined"); \
	} \
	__indice_bit = 0; \
	__indice_bloc = 0; \
	__taille_bloc = 0; \
	__type = 0; \
	__masque = 0; \
	{

#define END \
		strcpy((char *) __function_name, ""); \
		if (__constante != 0) \
			systemError(__LINE__, "Constant definition error"); \
	} } \
	return;

/*
--------------------------------------------------------------------------------
  Dclaration des fonctions externes
--------------------------------------------------------------------------------
*/

#define declareExternalFunction(name) \
	void name(struct_rpl_arguments *rpl_arguments) { \
	char __function_name[] = #name;

#define useExternalFunction(function) \
	void function(struct_rpl_arguments *rpl_arguments)

#define libraryName(name) char __library_name[] = #name;

#define exportExternalFunctions(...) \
	char **__external_symbols(unsigned long *nb_symbols) { \
		char arguments[] = #__VA_ARGS__; \
		char **tableau; \
		char *ptr1, *ptr2; \
		int drapeau; \
		unsigned long i; \
		(*nb_symbols) = 0; ptr1 = arguments; drapeau = 0; \
		while((*ptr1) != 0) \
		{ \
			if (((*ptr1) != ',') && ((*ptr1) != ' ')) drapeau = -1; \
			ptr1++; \
		} \
		if (drapeau == 0) return(NULL); \
		ptr1 = arguments; (*nb_symbols) = 1; \
		while((*ptr1) != 0) if ((*ptr1++) == ',') (*nb_symbols)++; \
		if ((tableau = malloc((*nb_symbols) * sizeof(char *))) == NULL) \
			return(NULL); \
		ptr2 = arguments; i = 0; \
		while(*ptr2 != 0) \
		{ \
			while(((*ptr2) == ' ') || ((*ptr2) == ',')) ptr2++; \
			ptr1 = ptr2; \
			while(((*ptr2) != 0) && ((*ptr2) != ',') && ((*ptr2) != ' ')) \
					ptr2++; \
			if ((tableau[i] = malloc((ptr2 + 2 + \
					strlen(__library_name) - ptr1) * \
					sizeof(unsigned char))) == NULL) \
				return(NULL); \
			sprintf(tableau[i], "%s$", __library_name); \
			strncat(&tableau[i][strlen(tableau[i])], ptr1, ptr2 - ptr1); \
			i++; \
			if ((*ptr2) != 0) \
			{ \
				while((*ptr2) == ' ') ptr2++; \
				if ((*ptr2) == ',') ptr2++; \
			} \
		} \
		(*nb_symbols) = i; \
		return(tableau); \
	}

#define endExternalFunction return; }

#define callExternalFunction(function) { \
	__taille_bloc = sizeof(t_8_bits) * 8; \
	__indice_bloc = (35 - 1) / __taille_bloc; \
	__indice_bit = (35 - 1) % __taille_bloc; \
	__masque = ((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1); \
	__evaluation = ((*rpl_arguments).drapeaux_etat[__indice_bloc] & __masque) \
			? d_vrai : d_faux; \
	__masque = ~(((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1)); \
	(*rpl_arguments).drapeaux_etat[__indice_bloc] &= __masque; \
	function(rpl_arguments); \
	if (__evaluation == d_vrai) \
	{ \
		__masque = ((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1); \
		(*rpl_arguments).drapeaux_etat[__indice_bloc] |= __masque; \
	} \
	else \
	{ \
		__masque = ~(((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1)); \
		(*rpl_arguments).drapeaux_etat[__indice_bloc] &= __masque; \
	} }

/*
--------------------------------------------------------------------------------
  Macros spcifiques  l'en-tte
--------------------------------------------------------------------------------
*/

#define declareHelpString(h) { \
	__presence_aide = d_vrai; \
	if ((*rpl_arguments).affichage_arguments == 'Y') \
	{ \
		printf("%s\n", h); \
		return; \
	} }

#define declareSymbolicConstant { \
	numberOfArguments(0); \
	(*rpl_arguments).constante_symbolique = 'Y'; \
	__constante++; }

#define numberOfArguments(n) { \
	__presence_validation = d_vrai; \
	if ((*rpl_arguments).test_instruction == 'Y') \
	{ \
		(*rpl_arguments).nombre_arguments = n; \
		__validation_instruction = d_vrai; \
	} \
	else \
	{ \
		__taille_bloc = sizeof(t_8_bits) * 8; \
		__indice_bloc = (31 - 1) / __taille_bloc; \
		__indice_bit = (31 - 1) % __taille_bloc; \
		__masque = ((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1); \
		if (((*rpl_arguments).drapeaux_etat[__indice_bloc] & __masque) != 0) \
		{ \
			(*rpl_arguments).l_base_pile_last = \
					sauvegarde_arguments(rpl_arguments, n); \
		} \
	} }

/*
--------------------------------------------------------------------------------
  Gestion des erreurs
--------------------------------------------------------------------------------
*/

#define returnOnError(...) { \
	if ((*rpl_arguments).erreur != 0) \
	{ \
		__VA_ARGS__; \
		return; \
	} }

#define systemError(n, message) { \
	(*rpl_arguments).erreur = n; \
	(*rpl_arguments).type_erreur = 'S'; \
	(*rpl_arguments).message_erreur = (unsigned char *) message; \
	return; }

#define executionError(n, message) { \
	(*rpl_arguments).erreur = n; \
	(*rpl_arguments).type_erreur = 'E'; \
	(*rpl_arguments).message_erreur = (unsigned char *) message; }

/*
--------------------------------------------------------------------------------
  Gestion de la pile oprationnelle
--------------------------------------------------------------------------------
*/

#define pushOnStack(object) { \
	if (((*rpl_arguments).l_base_pile = \
			empilement_pile_operationnelle(rpl_arguments, object)) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	}

#define pullFromStack(object, ...) { \
	(*rpl_arguments).l_base_pile = \
			depilement_pile_operationnelle(rpl_arguments, &object); \
	if (object == NULL) \
	{ \
		executionError(__LINE__, "Too few arguments"); \
	} \
	else if (strlen(#__VA_ARGS__) == 0) \
		systemError(__LINE__, "Undefined type"); \
	__type = 0; \
	if (strstr(#__VA_ARGS__, "integer") != NULL) \
		if (strcmp((char *) (*object).type, "INT") == 0) __type = 1; \
	if (strstr(#__VA_ARGS__, "real") != NULL) \
		if (strcmp((char *) (*object).type, "REL") == 0) __type = 1; \
	if (strstr(#__VA_ARGS__, "complex") != NULL) \
		if (strcmp((char *) (*object).type, "CPL") == 0) __type = 1; \
	if (__type == 0) \
	{ \
		freeObject(object); \
		executionError(__LINE__, "Type not allowed"); \
	} }

/*
--------------------------------------------------------------------------------
  Gestion des objets
--------------------------------------------------------------------------------
*/

#define then {
#define endIf } else { executionError(__LINE__, "Nothing to do"); }

//	Constantes symboliques

#define createSymbolicConstant(object, type, value) { \
	if ((strcmp(#type, "integer") != 0) && (strcmp(#type, "real") != 0)) \
		systemError(__LINE__, "Type not allowed for symbolic constant"); \
	__taille_bloc = sizeof(t_8_bits) * 8; \
	__indice_bloc = (35 - 1) / __taille_bloc; \
	__indice_bit = (35 - 1) % __taille_bloc; \
	__masque = ((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1); \
	if (((*rpl_arguments).drapeaux_etat[__indice_bloc] & __masque) != 0) \
	{ \
		createNameObject(object); \
		{ \
			char *__constant_name; \
			if ((__constant_name = malloc((strlen(__library_name) + \
					strlen(__function_name) + 2) * sizeof(char))) == NULL) \
				systemError(__LINE__, "Memory allocation error"); \
			sprintf(__constant_name, "%s$%s", __library_name, \
					__function_name); \
			setName(object, __constant_name); \
			free(__constant_name); \
		} \
	} \
	else \
	{ \
		if (strcmp(#type, "integer") == 0) \
		{ \
			createIntegerObject(object); \
			setInteger(object, value); \
		} \
		else if (strcmp(#type, "real") == 0) \
		{ \
			createRealObject(object); \
			setReal(object, value); \
		} \
	} \
	__constante--; }

#define createSymbolicComplexConstant(object, rp, ip) { \
	__taille_bloc = sizeof(t_8_bits) * 8; \
	__indice_bloc = (35 - 1) / __taille_bloc; \
	__indice_bit = (35 - 1) % __taille_bloc; \
	__masque = ((t_8_bits) 1) << (__taille_bloc - __indice_bit - 1); \
	if (((*rpl_arguments).drapeaux_etat[__indice_bloc] & __masque) != 0) \
	{ \
		createNameObject(object); \
		{ \
			char *__constant_name; \
			if ((__constant_name = malloc((strlen(__library_name) + \
					strlen(__function_name) + 2) * sizeof(char))) == NULL) \
				systemError(__LINE__, "Memory allocation error"); \
			sprintf(__constant_name, "%s$%s", __library_name, \
					__function_name); \
			setName(object, __constant_name); \
			free(__constant_name); \
		} \
	} \
	else \
	{ \
		createComplexObject(object); \
		setComplex(object, rp, im); \
	} \
	__constante--; }

//	Integer

#define setInteger(object, value) { \
	ifIsInteger(object) (*((integer8 *) (*object).objet)) = (integer8) value; \
	else systemError(__LINE__, "Type mistmatch error"); }

#define isInteger(object) \
	(strcmp((char *) (*object).type, "INT") == 0)

#define ifIsInteger(object) if (isInteger(object))
#define elseIfIsInteger(object) } else ifIsInteger(object)

#define getInteger(object, value) { \
	ifIsInteger(object) value = (*((integer8 *) (*object).objet)); \
	else systemError(__LINE__, "Type mismatch error"); }

#define createIntegerObject(object) { \
	if (object != NULL) \
		systemError(__LINE__, "Reallocated object"); \
	if ((object = malloc(sizeof(struct_objet))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).type, "INT");\
	if (((*object).objet = malloc(sizeof(integer8))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	setInteger(object, 0); }

// Real

#define setReal(object, value) { \
	ifIsReal(object) (*((real8 *) (*object).objet)) = (real8) value; \
	else systemError(__LINE__, "Type mistmatch error"); }

#define isReal(object) \
	(strcmp((char *) (*object).type, "REL") == 0)

#define ifIsReal(object) if (isReal(object))
#define elseIfIsReal(object) } else ifIsReal(object)

#define getReal(object, value) { \
	ifIsReal(object) value = (*((real8 *) (*object).objet)); \
	else systemError(__LINE__, "Type mismatch error"); }

#define createRealObject(object) { \
	if (object != NULL) \
		systemError(__LINE__, "Reallocated object"); \
	if ((object = malloc(sizeof(struct_objet))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).type, "REL");\
	if (((*object).objet = malloc(sizeof(real8))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	setReal(object, 0); }

// Complex

#define setComplex(object, rp, ip) { \
	ifIsComplex(object) \
	{ \
		setRealPartOfComplex(object,rp); \
		setImaginaryPartOfComplex(object, ip); \
	} \
	else systemError(__LINE__, "Type mismatch error"); }

#define setRealPartOfComplex(object, value) { \
	ifIsComplex(object) (*((complex16 *) (*object).objet)).partie_reelle = \
			value; \
	else systemError(__LINE__, "Type mismatch error"); }

#define setImaginaryPartOfComplex(object, value) \
	(*((complex16 *) (*object).objet)).partie_imaginaire = value

#define getRealPartOfComplex(object, value) \
	value = (*((complex16 *) (*object).objet)).partie_reelle

#define getImaginaryPartOfComplex(object, value) \
	value = (*((complex16 *) (*object).objet)).partie_imaginaire

#define isComplex(object) \
	(strcmp((char *) (*object).type, "CPL") == 0)

#define ifIsComplex(object) if (isComplex(object))
#define elseIfIsComplex(object) } else ifIsComplex(object)

#define getComplex(object, value) { \
	ifIsComplex(object) value = (*((complex16 *) (*object).objet)); \
	else systemError(__LINE__, "Not a complex"); }

#define createComplexObject(object) { \
	if (object != NULL) \
		systemError(__LINE__, "Reallocated object"); \
	if ((object = malloc(sizeof(struct_objet))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).type, "CPL");\
	if (((*object).objet = malloc(sizeof(real8))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	setComplex(object, 0, 0); }

// Integer vector

#define createIntegerVectorObject

#define createRealVectorObject

#define createComplexVectorObject

#define createIntegerMatrixObject

#define createRealMatrixObject

#define createComplexMatrixObject

#define createBinaryIntegerObject

// Name

#define isName(object) \
	(strcmp((char *) (*object).type, "NOM") == 0)

#define ifIsName(object) if (isName(object))
#define elseIfIsName(object) else if (isName(object))

#define setName(object, value) { \
	ifIsName(object) \
	{ \
		free((*((struct_nom *) (*object).objet)).nom); \
		(*((struct_nom *) (*object).objet)).symbole = d_faux; \
		if (((*((struct_nom *) (*object).objet)).nom = malloc( \
				(strlen(#value) + 1) * sizeof(unsigned char))) == NULL) \
			systemError(__LINE__, "Memory allocation error"); \
		strcpy((char *) (*((struct_nom *) (*object).objet)).nom, \
				(char *) value); \
	} \
	else systemError(__LINE__, "Type mistmatch error"); }

#define createNameObject(object) { \
	if (object != NULL) \
		systemError(__LINE__, "Reallocated object"); \
	if ((object = malloc(sizeof(struct_objet))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).type, "NOM"); \
	if (((*object).objet = malloc(sizeof(struct_nom))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	(*((struct_nom *) (*object).objet)).symbole = d_faux; \
	if (((*((struct_nom *) (*object).objet)).nom = malloc( \
			sizeof(unsigned char))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*((struct_nom *) (*object).objet)).nom, ""); }

// String

#define isString(object) \
	(strcmp((char *) (*object).type, "CHN") == 0)

#define ifIsString(object) if (isString(object))
#define elseIfIsString(object) else if (isString(objet))

#define setString(object, string) { \
	ifIsString(object) \
	{ \
		free((unsigned char *) (*object).objet); \
		if (((*object).objet = malloc((strlen(string) + 1) * \
				sizeof(unsigned char))) == NULL) \
			systemError(__LINE__, "Memory allocation error"); \
		strcpy((char *) (*object).objet, string); \
	} }

#define createStringObject(objet) { \
	if (object != NULL) \
		systemError(__LINE__, "Reallocated object"); \
	if ((object = malloc(sizeof(struct_objet))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).type, "CHN"); \
	if (((*object).objet = malloc((strlen(string) + 1) * \
			sizeof(unsigned char))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	if (((*object).objet = malloc(sizeof(unsigned char))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).objet, ""); }

// List

#define isList(object) \
	(strcmp((char *) (*object).type, "LST") == 0)

#define ifIsList(object) if (isList(object))
#define elseIfIsList(object) else if (isList(object))

#define createListObject(object) { \
	if (object != NULL) \
		systemError(__LINE__, "Reallocated object"); \
	if ((object = malloc(sizeof(struct_objet))) == NULL) \
		systemError(__LINE__, "Memory allocation error"); \
	strcpy((char *) (*object).type, "LST"); \
	(*object).objet = NULL; }

#define addObjectToList(list, object) { \
	ifIsList(list) \
	{ \
		if ((*list).objet == NULL) \
		{ \
			if (((*list).objet = malloc(sizeof(struct_liste_chainee))) \
					== NULL) \
				systemError(__LINE__, "Memory allocation error"); \
			(*((*struct_list_chainee *) (*list).objet)).suivant = NULL; \
			(*((*struct_list_chainee *) (*list).objet)).donnee = object; \
		} \
		else \
		{ \
			struct_liste_chainee	*l_element_courant; \
			l_element_courant = (*list).objet; \
			while((*l_element_courant).suivant != NULL) \
				l_element_courant = (*l_element_courant).suivant; \
			if (((*l_element_courant).suivant = \
					malloc(sizeof(struct_liste_chainee))) == NULL) \
				systemError(__LINE__, "Memory allocation error"); \
			l_element_courant = (*l_element_courant).suivant; \
			(*l_element_courant).suivant = NULL; \
			(*l_element_courant).donnee = object; \
		}\
	} \
	else systemError(__LINE__, "Type mistmatch error"); }

#define removeObjectFromList(list, object) \
	ifIsList(list) \
	{ \
		if ((*object).objet == NULL) \
			{}\
	} \
	else systemError(__LINE__, "Type mistmatch error"); }

#define freeObject(object) \
	{ \
		if (object == NULL) \
			systemError(__LINE__, "Nullified object"); \
		liberation(object); \
		object = NULL; \
	}

#endif

// vim: ts=4
